@@ -1,4 +1,5 @@
 use std::fs::{File, OpenOptions};
+use std::hash::{Hash, Hasher};
 use std::io;
 use std::mem;
 use std::os::windows::fs::OpenOptionsExt;
@@ -14,7 +15,6 @@ use winapi::winbase::{
     FILE_FLAG_BACKUP_SEMANTICS,
     STD_INPUT_HANDLE, STD_OUTPUT_HANDLE, STD_ERROR_HANDLE,
 };
-use winapi::winerror::ERROR_INVALID_HANDLE;
 
 // For correctness, it is critical that both file handles remain open while
 // their attributes are checked for equality. In particular, the file index
@@ -25,7 +25,7 @@ use winapi::winerror::ERROR_INVALID_HANDLE;
 // https://msdn.microsoft.com/en-us/library/windows/desktop/aa363788(v=vs.85).aspx
 //
 // It gets worse. It appears that the index numbers are not always
-// guaranteed to be unqiue. Namely, ReFS uses 128 bit numbers for unique
+// guaranteed to be unique. Namely, ReFS uses 128 bit numbers for unique
 // identifiers. This requires a distinct syscall to get `FILE_ID_INFO`
 // documented here:
 // https://msdn.microsoft.com/en-us/library/windows/desktop/hh802691(v=vs.85).aspx
@@ -66,7 +66,7 @@ pub struct Handle {
     key: Option<Key>,
 }
 
-#[derive(Debug, Eq, PartialEq)]
+#[derive(Debug, Eq, PartialEq, Hash)]
 struct Key {
     volume: DWORD,
     idx_high: DWORD,
@@ -76,6 +76,8 @@ struct Key {
 impl Drop for Handle {
     fn drop(&mut self) {
         if self.is_std {
+            // unwrap() will not panic. Since we were able to open an
+            // std stream successfully, then `file` is guaranteed to be Some()
             self.file.take().unwrap().into_raw_handle();
         }
     }
@@ -94,23 +96,33 @@ impl PartialEq for Handle {
 
 impl AsRawHandle for ::Handle {
     fn as_raw_handle(&self) -> RawHandle {
+        // unwrap() will not panic. Since we were able to open the
+        // file successfully, then `file` is guaranteed to be Some()
         self.0.file.as_ref().take().unwrap().as_raw_handle()
     }
 }
 
 impl IntoRawHandle for ::Handle {
     fn into_raw_handle(mut self) -> RawHandle {
+        // unwrap() will not panic. Since we were able to open the
+        // file successfully, then `file` is guaranteed to be Some()
         self.0.file.take().unwrap().into_raw_handle()
     }
 }
 
+impl Hash for Handle {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        self.key.hash(state);
+    }
+}
+
 impl Handle {
     pub fn from_path<P: AsRef<Path>>(p: P) -> io::Result<Handle> {
-        let file = try!(OpenOptions::new()
+        let file = OpenOptions::new()
             .read(true)
             // Necessary in order to support opening directory paths.
             .custom_flags(FILE_FLAG_BACKUP_SEMANTICS)
-            .open(p));
+            .open(p)?;
         Handle::from_file(file)
     }
 
@@ -119,19 +131,15 @@ impl Handle {
     }
 
     fn from_std_handle(file: File) -> io::Result<Handle> {
-        let err = match file_info(&file) {
-            Ok(info) => return Ok(Handle::from_file_info(file, true, info)),
-            Err(err) => err,
-        };
-        if err.raw_os_error() == Some(ERROR_INVALID_HANDLE as i32) {
+        match file_info(&file) {
+            Ok(info) => Ok(Handle::from_file_info(file, true, info)),
             // In a Windows console, if there is no pipe attached to a STD
             // handle, then GetFileInformationByHandle will return an error.
             // We don't really care. The only thing we care about is that
             // this handle is never equivalent to any other handle, which is
             // accomplished by setting key to None.
-            return Ok(Handle { file: Some(file), is_std: true, key: None });
+            Err(_) => Ok(Handle { file: Some(file), is_std: true, key: None }),
         }
-        Err(err)
     }
 
     fn from_file_info(
@@ -169,10 +177,14 @@ impl Handle {
     }
 
     pub fn as_file(&self) -> &File {
+        // unwrap() will not panic. Since we were able to open the
+        // file successfully, then `file` is guaranteed to be Some()
         self.file.as_ref().take().unwrap()
     }
 
     pub fn as_file_mut(&mut self) -> &mut File {
+        // unwrap() will not panic. Since we were able to open the
+        // file successfully, then `file` is guaranteed to be Some()
         self.file.as_mut().take().unwrap()
     }
 }
